#-----------------------------------------------------------------------
# Modified 2003 Boris Schfer <menc@noid-project>
# for the Video::Menc project
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
# 
# This module is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::Menc::GUI::Project;

use base Video::Menc::GUI::Component;

use Video::Menc::GUI::ImageClip;
use Video::Menc::GUI::Progress;

# These are not classes. they define methods inside
# the Video::Menc::GUI::Project package. This is only for
# splitting this huge package into handy pieces.

use Video::Menc::GUI::Project::TitleTab2;
use Video::Menc::GUI::Project::TitleTab;
use Video::Menc::GUI::Project::ClipZoomTab;
use Video::Menc::GUI::Project::MencoderTab;

use Carp;
use strict;

sub project              { shift->{project}          }
sub set_project          { shift->{project}          = $_[1] }

sub selected_title       { shift->{selected_title}   }
sub set_selected_title   { shift->{selected_title}   = $_[1] }

sub gtk_title_labels     { shift->{gtk_title_labels} }  # lref
sub set_gtk_title_labels { shift->{gtk_title_labels} = $_[1] }

sub adjust_widgets       { shift->{adjust_widgets}   }  # href
sub set_adjust_widgets   { shift->{adjust_widgets}   = $_[1] }

#------------------------------------------------------------------------
# Build Project GUI
#------------------------------------------------------------------------

sub build {
  my $self = shift; $self->trace_in;

  my $vbox = Gtk::VBox->new;
  $vbox->set_border_width(5);
  $vbox->show;

  my $notebook = Gtk::Notebook->new;
  $notebook->set_tab_pos ('top');
  $notebook->set_usize (undef, 100);
  $notebook->set_homogeneous_tabs(1);
  $notebook->show;
  
  $self->set_gtk_title_labels([]);
  $self->set_adjust_widgets({});

  my $label;

  $label = Gtk::Label->new (" File ");
  $notebook->append_page ($self->create_title2_tab, $label);

  $label = Gtk::Label->new (" DVD ");
  $notebook->append_page ($self->create_title_tab, $label);

  $label = Gtk::Label->new (" Scale & Crop ");
  $notebook->append_page ($self->create_adjust_tab, $label);

  $label = Gtk::Label->new (" Mencoder ");
  $notebook->append_page ($self->create_mencoder_tab, $label);

  $vbox->pack_start ($notebook, 1, 1, 0);

  my $frame = Gtk::Frame->new ("Status");
  $frame->show;
  $vbox->pack_start ($frame, 0, 1, 0);

  my $hbox = Gtk::HBox->new;
  $hbox->set_border_width(5);
  $hbox->show;
  $frame->add($hbox);

  my $progress = Video::Menc::GUI::Progress->new;
  $progress->build;

  $hbox->pack_start($progress->widget,1,1,0);

  $self->set_widget($vbox);
  $self->set_comp ( project => $self );

  return $vbox;
}

sub fill_with_values {
  my $self = shift;

  $self->init_title_labels;
  $self->init_audio_popup;
  $self->init_audio_popup2;
  $self->init_sub_popup;
  $self->init_adjust_values;
  $self->init_mencoder_values;
  
  1;
}

sub init_title_labels {
  my $self = shift; $self->trace_in;

  my $title = $self->selected_title;

  return if not $title;

    my $title_string;

    if ( $title->type eq undef ) {
    $title_string = "";    
  } 
  if ( $title->type eq "dvd" ) {
    my $audio_label;
    my $audio_label2;
    my $audio_channel = $title->audio_channel;
    my $audio2_channel = $title->audio2_channel;
    if ( @{$title->audio_tracks} >= 1 ) {
      if ( $audio_channel >= 0 ) {
        my $audio = $title->audio_tracks->[$audio_channel];
        my $bitrate = $audio->{bitrate};
        $bitrate = $bitrate/1000 if ($bitrate);
        $bitrate = " ($bitrate kbits/s)" if ($bitrate);
      
        $audio_label =
          "Audio Channel: ".
          "$audio->{lang} $audio->{format}".
          "$bitrate";
        
        $title->set_audiocmd(" -aid $audio->{id} ");
      } else {
        $audio_label = "No Audio";
        $title->set_audiocmd(" -nosound ");
      }
      if ( $audio2_channel >= 0 ) {
        my $audio = $title->audio_tracks->[$audio2_channel];
        my $bitrate = $audio->{bitrate};
        $bitrate = $bitrate/1000 if ($bitrate);
        $bitrate = " ($bitrate kbits/s)" if ($bitrate);
      
        $audio_label2 =
          "2nd Audio Channel: ".
          "$audio->{lang} $audio->{format}".
          "$bitrate";
        
        $title->set_audiocmd2(" -aid $audio->{id} ");
      } else {
        $audio_label2 = "No 2nd Audio";
        $title->set_audiocmd2(" -nosound ");
      }
    } else {
        $audio_label = "No Audio";
      $audio_label2 = "No 2nd Audio";
      $title->set_audiocmd(" -nosound ");
      $title->set_audiocmd2(" -nosound ");
    }    

    my $nr = $title->dvdtrack;
    
    $title_string = "DVD Title #$nr - ".
                  $self->get_title_info (title => $title).
                  "\n".$audio_label."\n".$audio_label2;
  } 
  if ( $title->type eq "file" ) {
    my $filename = $title->filename;
    my $audio_label;
    my $audio_label2;
    my $audio_channel = $title->audio_channel;
    my $audio2_channel = $title->audio2_channel;
    if ( $title->audio_tracks ) {
      if ( @{$title->audio_tracks} >= 1 ) {
        if ( $audio_channel >= 0 ) {
          my $audio = $title->audio_tracks->[$audio_channel];
          my $bitrate = $audio->{bitrate};
          $bitrate = $bitrate/1000 if ($bitrate);
          $bitrate = " $bitrate kbits/s" if ($bitrate);
      
          $audio_label =
            "Audio Channel: ".
            "$audio->{format}$bitrate (Id=$audio->{id})";
        
          $title->set_audiocmd(" -aid $audio->{id} ");
        } else {
          $audio_label = "No Audio";
          $title->set_audiocmd(" -nosound ");
        }
        if ( $audio2_channel >= 0 ) {
          my $audio = $title->audio_tracks->[$audio2_channel];
          my $bitrate = $audio->{bitrate};
          $bitrate = $bitrate/1000 if ($bitrate);
          $bitrate = " $bitrate kbits/s" if ($bitrate);
      
          $audio_label2 =
            "2nd Audio Channel: ".
            "$audio->{format}$bitrate (Id=$audio->{id})";
        
          $title->set_audiocmd2(" -aid $audio->{id} ");
        } else {
          $audio_label2 = "No 2nd Audio";
          $title->set_audiocmd2(" -nosound ");
        }        
        
      } else {
        $audio_label = "No Audio";
        $audio_label2 = "No 2nd Audio";
        $title->set_audiocmd(" -nosound ");
        $title->set_audiocmd2(" -nosound ");
      }
    } else {
      $audio_label = "No Audio";
      $audio_label2 = "No 2nd Audio";
      $title->set_audiocmd(" -nosound ");
      $title->set_audiocmd2(" -nosound ");
    }    
    
    my $widgets = $self->adjust_widgets;
    my $info = $self->get_title_info (title => $title);  
      
    my $length = $title->runtime;
    my $h = int($length/3600);
    my $m = int(($length-$h*3600)/60);
    my $s = $length-$h*3600-$m*60;
    my $length2 = sprintf ("%02d:%02d:%02d", $h, $m, $s);
      
    my $fileinfo = "${length}sec ($length2), $info";        
    
    $title_string = "File '$filename' $fileinfo".
            "\n".$audio_label."\n".$audio_label2;
  }

  foreach my $label ( @{$self->gtk_title_labels} ) {
    $label->set_text ( $title_string );
  }

  1;
}

sub get_title_info {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($title) = @par{'title'};
  
  my $fps = $title->fps;
  $fps =~ s/\.0+$//;
  
  my $aspect = $title->aspect;
  $aspect = "4:3" if ( $aspect == 1.3333 );
  $aspect = "16:9" if ( $aspect == 1.7778 );

  return $title->width."x".$title->height.", ".
         "$fps fps, ".
         $aspect;
}

sub create_selected_title {
  my $self = shift; $self->trace_in;

  my $frame = Gtk::Frame->new ("Selected Title");
  $frame->show;

  my $hbox = Gtk::HBox->new;
  $hbox->set_border_width(5);
  $hbox->show;

  my $label = Gtk::Label->new;
  $label->show;
  $label->set_justify('left');

  $hbox->pack_start ( $label, 0, 1, 0);

  $frame->add ($hbox);

  push @{$self->gtk_title_labels}, $label;

  return $frame;
}

1;
