#-----------------------------------------------------------------------
# Modified 2003 Boris Schfer <menc@noid-project>
# for the Video::Menc project
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
# 
# This module is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::Menc::GUI::Project;

use Carp;
use strict;

#------------------------------------------------------------------------
# Build Adjustments Tab
#------------------------------------------------------------------------

sub create_adjust_tab {
  my $self = shift; $self->trace_in;

  my $vbox = Gtk::VBox->new;
  $vbox->set_border_width(5);
  $vbox->show;
  
  my $selected_title = $self->create_selected_title;
  $vbox->pack_start ( $selected_title, 0, 1, 0);

  my ($label, $entry, $popup_menu, $popup, $item,
      $box, $image, $button, $frame, $hbox);

  # preview images -------------------------------------------------

  my $title = $self->selected_title;

  $frame = Gtk::Frame->new ("Preview Images");
  $frame->show;
  $vbox->pack_start ( $frame, 0, 1, 0);

  $box = Gtk::VBox->new;
  $box->show;
  $frame->add ($box);
  
  # preview frame
  $hbox = Gtk::HBox->new;
  $hbox->set_border_width(5);
  $hbox->show;
  $box->pack_start($hbox, 0, 1, 0);

  $label = Gtk::Label->new ("Grab Preview (seconds)");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  
  $entry = Gtk::Entry->new;
  $entry->set_usize(80, undef);
  $entry->show;
  $hbox->pack_start($entry, 0, 1, 0);
  $self->adjust_widgets->{preview_second} = $entry;

  $button = Gtk::Button->new_with_label (" Grab Frame ");
  $button->show;
  $button->signal_connect ("clicked", sub { 
    $self->grab_preview_frame;
    $self->get_sizes;
  } );
  $hbox->pack_start($button, 0, 1, 0);

  # images

  $hbox = Gtk::HBox->new (0, 5);
  $hbox->set_border_width(5);
  $hbox->show;
  $box->pack_start($hbox, 0, 1, 0);

  # Original

  $box = Gtk::VBox->new;
  $box->show;
  $hbox->pack_start($box, 0, 1, 0);
  
  $frame = Gtk::Frame->new;
  $frame->show;
  $box->pack_start ($frame, 0, 1, 0);
  
  $image = Video::Menc::GUI::ImageClip->new (
    gtk_window => $self->comp('main')->widget,
    width      => 270,
    height     => 160,
    thumbnail  => $self->config('thumbnail_factor'),
    no_clip    => 1,
  );
  $frame->add ($image->widget);
  $image->widget->signal_connect (
    'button_press_event', sub {
      $self->open_preview_window (
        type => 'orig'
      );
    }
  );

  $self->adjust_widgets->{image_orig} = $image;

  $label = Gtk::Label->new ("Original");
  $label->show;
  $box->pack_start ($label, 0, 1, 0);
  $label = Gtk::Label->new ("");
  $label->show;
  $box->pack_start ($label, 0, 1, 0);
  $self->adjust_widgets->{orig_info_label} = $label;

  # Scale

  $box = Gtk::VBox->new;
  $box->show;
  $hbox->pack_start($box, 0, 1, 0);
  
  $frame = Gtk::Frame->new;
  $frame->show;
  $box->pack_start ($frame, 0, 1, 0);
  
  $image = Video::Menc::GUI::ImageClip->new (
    gtk_window => $self->comp('main')->widget,
    width      => 270,
    height     => 160,
    thumbnail  => $self->config('thumbnail_factor'),
    no_clip    => 1,
  );
  $frame->add ($image->widget);
  $image->widget->signal_connect (
    'button_press_event', sub {
      $self->open_preview_window (
        type => 'scale'
      );
    }
  );

  $self->adjust_widgets->{image_scale} = $image;
  
  $label = Gtk::Label->new ("After Scale");
  $label->show;
  $box->pack_start ($label, 0, 1, 0);
  $label = Gtk::Label->new ("");
  $label->show;
  $box->pack_start ($label, 0, 1, 0);
  $self->adjust_widgets->{scale_info_label} = $label;

  # Crop

  $box = Gtk::VBox->new;
  $box->show;
  $hbox->pack_start($box, 0, 1, 0);
  
  $frame = Gtk::Frame->new;
  $frame->show;
  $box->pack_start ($frame, 0, 1, 0);
  
  $image = Video::Menc::GUI::ImageClip->new (
    gtk_window => $self->comp('main')->widget,
    width      => 270,
    height     => 160,
    thumbnail  => $self->config('thumbnail_factor'),
    no_clip    => 1,
  );
  $frame->add ($image->widget);
  $image->widget->signal_connect (
    'button_press_event', sub {
      $self->open_preview_window (
        type => 'crop'
      );
    }
  );

  $self->adjust_widgets->{image_crop} = $image;
  
  $label = Gtk::Label->new ("After Crop");
  $label->show;
  $box->pack_start ($label, 0, 1, 0);
  $label = Gtk::Label->new ("");
  $label->show;
  $box->pack_start ($label, 0, 1, 0);
  $self->adjust_widgets->{crop_info_label} = $label;

  # Adjust Scale and Crop Parameters ------------------------------

  $frame = Gtk::Frame->new ("Adjust Scale and Crop Parameters");
  $frame->show;
  $vbox->pack_start ( $frame, 0, 1, 0);

  $hbox = Gtk::HBox->new;
  $hbox->set_border_width(5);
  $hbox->show;

  $frame->add ($hbox);

  my $table = Gtk::Table->new ( 3, 3, 0 );
  $table->show;
  $table->set_row_spacings ( 10 );
  $table->set_col_spacings ( 10 );
  $hbox->pack_start ($table, 0, 1, 0);

    # Presets Menu
  my $row = 0;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Presets");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $popup_menu = Gtk::Menu->new;
  $popup_menu->show;

  my $presets = $self->config_object->presets;
  my $i = 0;
  foreach my $preset ( @{$presets} ) {
    $item = Gtk::MenuItem->new ($preset->title);
    $item->show;
    $item->signal_connect ("select", sub {
      my ($widget, $name) = @_;
      my $title = $self->selected_title;
      return 1 if not $title;
      $title->set_preset ($name);
    }, $preset->name);
    $popup_menu->append($item);
    ++$i;
  }

  $popup = Gtk::OptionMenu->new;
  $popup->show;
  $popup->set_menu($popup_menu);
  
  $table->attach_defaults ($popup, 1, 2, $row, $row+1);

  $self->adjust_widgets->{preset_popup}      = $popup;
  $self->adjust_widgets->{preset_popup_menu} = $popup_menu;

  $button = Gtk::Button->new_with_label ("Apply Preset Values");
  $button->show;
  $button->signal_connect ("clicked", sub {
    my $title = $self->selected_title;
    return 1 if not $title;
    my $preset = $self->config_object->get_preset (
      name => $title->preset
    );
    return if not $preset;
    $title->apply_preset ( preset => $preset );
    $self->make_previews;
    $self->get_sizes;
    $self->fill_with_values;
  });
  $table->attach_defaults ($button, 2, 3, $row, $row+1);
  
  # Scale
    $row++;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Scale");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $hbox = Gtk::HBox->new;
  $hbox->show;
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  $label = Gtk::Label->new("Width");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->set_usize(40, undef);
  $entry->show;
  $hbox->pack_start($entry, 0, 1, 0);
  $self->adjust_widgets->{tc_scale_width}      = $entry;

  $label = Gtk::Label->new("and/or Height");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->set_usize(40, undef);
  $entry->show;
  $hbox->pack_start($entry, 0, 1, 0);
  $self->adjust_widgets->{tc_scale_height}      = $entry;

  $button = Gtk::Button->new_with_label (" Generate Preview Images ");

  $button->show;
  $button->signal_connect ("clicked", sub {
    $self->make_previews;
    $self->show_preview_images;
    $self->get_sizes;
    $self->fill_with_values;
  });
  $table->attach_defaults ($button, 2, 3, $row, $row+1);

  # Crop
  $row++;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Crop");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $hbox = Gtk::HBox->new;
  $hbox->show;
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  $label = Gtk::Label->new("Top");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->set_usize(40, undef);
  $entry->show;
  $hbox->pack_start($entry, 0, 1, 0);
  $self->adjust_widgets->{tc_crop_top}      = $entry;

  $label = Gtk::Label->new("Bottom");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->set_usize(40, undef);
  $entry->show;
  $hbox->pack_start($entry, 0, 1, 0);
  $self->adjust_widgets->{tc_crop_bottom}   = $entry;

  $label = Gtk::Label->new("Left");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->set_usize(40, undef);
  $entry->show;
  $hbox->pack_start($entry, 0, 1, 0);
  $self->adjust_widgets->{tc_crop_left}     = $entry;

  $label = Gtk::Label->new("Right");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->set_usize(40, undef);
  $entry->show;
  $hbox->pack_start($entry, 0, 1, 0);
  $self->adjust_widgets->{tc_crop_right}      = $entry;

  # Separator
  ++$row;
  my $sep = Gtk::HSeparator->new;
  $sep->show;
  $table->attach_defaults ($sep, 0, 2, $row, $row+1);  

  # Re-Scale
  $row++;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Re-Scale to");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $hbox = Gtk::HBox->new;
  $hbox->show;
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  $label = Gtk::Label->new("Width");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->set_usize(40, undef);
  $entry->show;
  $hbox->pack_start($entry, 0, 1, 0);
  $self->adjust_widgets->{tc_rescale_width}      = $entry;

  $label = Gtk::Label->new(" and/or Height");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->set_usize(40, undef);
  $entry->show;
  $hbox->pack_start($entry, 0, 1, 0);
  $self->adjust_widgets->{tc_rescale_height}      = $entry;

  $button = Gtk::Button->new_with_label (" Re-Scale ");

  $button->show;
  $button->signal_connect ("clicked", sub {
    $self->rescale;
  });
  $table->attach_defaults ($button, 2, 3, $row, $row+1);
  
  # connect changed signals
  my $widgets = $self->adjust_widgets;
  foreach my $attr (qw ( preview_second
                         tc_scale_width tc_scale_height
                         tc_crop_top  tc_crop_bottom
                         tc_crop_left tc_crop_right )) {
    $widgets->{$attr}->signal_connect ("changed", sub {
      my ($widget, $method) = @_;
      $self->selected_title->$method ( $widget->get_text );
    }, "set_$attr");
  }

  return $vbox;
}

sub get_sizes {
  my $self = shift; $self->trace_in;
  
  my $title = $self->selected_title;
  if ($title) {      
    my $width = $title->width;
    my $height = $title->height;
    #my $aspect = $title->aspect;
    my $scalew = $title->tc_scale_width;
    my $scaleh = $title->tc_scale_height;
    my $cropt = $title->tc_crop_top;
    my $cropb = $title->tc_crop_bottom;
    my $cropl = $title->tc_crop_left;
    my $cropr = $title->tc_crop_right;
      
    #if ($height > 0 and $aspect > 0) {
    #  $width = $aspect * $height;
    #  $width = $width / 8 + 0.5;
    #  $width = sprintf("%d", $width);
    #  $width = $width * 8;
    #}
  
    if (not $scalew and $scaleh) {
      $scalew = $scaleh / $height * $width;
      $scalew = $scalew / 8 + 0.5;
      $scalew = sprintf("%d", $scalew);
      $scalew = $scalew * 8;    
      $width = $scalew;
      $height = $scaleh;
    }
  
    if (not $scaleh and $scalew) {
      $scaleh = $scalew / $width * $height;
      $scaleh = $scaleh / 8 + 0.5;
      $scaleh = sprintf("%d", $scaleh);
      $scaleh = $scaleh * 8;        
      $width = $scalew;
      $height = $scaleh;  
    }  
    
    if ($scaleh and $scalew) {
      $width = $scalew;
      $height = $scaleh;
    }

    $width -= $cropl if ($cropl);
    $width -= $cropr if ($cropr);
    $height -= $cropt if ($cropt);
    $height -= $cropb if ($cropb);  
    
    $title->set_adjust_width($width);
    $title->set_adjust_height($height);
  }
}

sub init_adjust_values {
  my $self = shift; $self->trace_in;

  my $title = $self->selected_title;
  return 1 if not $title;
  return 1 if not $self->adjust_widgets->{preview_second};

  $self->show_preview_images;

  my $widgets = $self->adjust_widgets;
  foreach my $attr (qw ( preview_second
                         tc_scale_width tc_scale_height
                         tc_crop_top  tc_crop_bottom
                         tc_crop_left tc_crop_right )) {
    $widgets->{$attr}->set_text ($self->selected_title->$attr);
  }

  my $preset_name = $title->preset;
  my $i = 0;
  foreach my $preset ( @{$self->config_object->presets} ) {
    last if $preset_name eq $preset->name;
    ++$i;
  }
  $i = 0 if $i >= @{$self->config_object->presets};
  $widgets->{preset_popup}->set_history ($i);

  1;
}

sub show_preview_images {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($type) = @par{'type'};

  my $title = $self->selected_title;
  return 1 if not $title;
  
  my ($image, @types, $filename);

  if ( $type ) {
    push @types, $type;
  } else {
    @types = qw ( orig scale crop );
  }

  my ($width, $height, $text, $ratio);
  foreach $type ( @types ) {
    $image = $self->adjust_widgets->{"image_$type"};
    next if not $image;

    $filename = $title->preview_filename (type => $type);
    if ( not -f $filename ) {
      $image->set_gdk_pixbuf(undef);
      next;
    }

    $image->load_image (
      filename => $filename
    );
    $image->draw;
  }

  $self->show_preview_labels ( type => $type );
  
  1;
}

sub show_preview_labels {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($type) = @par{'type'};

  my $title = $self->selected_title;
  return 1 if not $title;
  
  my ($image, @types, $filename);

  if ( $type ) {
    push @types, $type;
  } else {
    @types = qw ( orig scale crop );
  }

  my ($width, $height, $warn_width, $warn_height, $text, $ratio);
  foreach $type ( @types ) {
    $image = $self->adjust_widgets->{"image_$type"};
    $width  = $image->image_width;
    $height = $image->image_height;
    next if $width == 0 or $height == 0;
    $ratio  = sprintf ("%2.2f", ($width/$height));
    $ratio  = "4:3" if $ratio eq '1.33';
    $ratio  = "16:9" if $ratio eq '1.77';
    $warn_width = ($width%8) ? "!":"";
    $warn_height = ($height%8) ? "!":"";
    $text = sprintf ("Size: %d%sx%d%s, Ratio: %s",
      $width, $warn_width, $height, $warn_height,
      $ratio
    );
    
    $self->adjust_widgets->{$type."_info_label"}->set_text($text);
  }
  
  1;
}

sub grab_preview_frame {
  my $self = shift; $self->trace_in;

  my $title = $self->selected_title;
  return 1 if not $title;
  return 1 if $self->comp('progress')->is_active;
  
  my $second = $title->preview_second;
  my $filename = $title->preview_filename( type => 'orig' );

  return 1 if not defined $second;

  my $fh = $title->take_snapshot_async_start (
    second    => $second,
    filename => $filename,
  );

  $self->comp('progress')->open_continious_progress (
    max_value => $second,
    label     => "Grabbing Preview Image",
    fh        => $fh,
    get_progress_callback => sub {
      my %par = @_;
      my ($buffer) = @par{'buffer'};
      $buffer =~ /\[0+-(\d+)\].*?$/;
      return $1;
    },
    finished_callback => sub {
      $title->take_snapshot_async_stop ( fh => $fh );
      $self->make_previews;
      $self->show_preview_images;
    },
    cancel_callback => sub {
      close $fh;
    },
  );
  
  1;
}

sub make_previews {
  my $self = shift; $self->trace_in;
  
  my $title = $self->selected_title;
  return 1 if not $title;

  $title->make_preview_scale;
  $title->make_preview_crop;
  
  1;
}

sub open_preview_window {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($type) = @par{'type'};
  
  return 1 if defined $self->adjust_widgets->{"window_$type"};
  my $title = $self->selected_title;
  return 1 if not $title;

  my $file_type;
  $file_type = 'orig' if $type eq 'orig';
  $file_type = 'scale' if $type eq 'scale';
  $file_type = 'scale' if $type eq 'crop';

  my $filename = $title->preview_filename (type => $file_type);
  return 1 if not -f $filename;

  my $win = Gtk::Window->new;
  $win->set_uposition (10,10);
  $win->signal_connect( 'destroy', sub {
    $self->adjust_widgets->{"window_$type"} = undef;
    $self->make_previews if $type ne 'scale';
    $self->show_preview_images;
    $self->get_sizes;
    $self->fill_with_values;
  } );
  $win->set_title ("Adjust $type");
  $win->show;
  
  my $vbox = Gtk::VBox->new;
  $vbox->show;
  $win->add($vbox);

  my $hbox = Gtk::HBox->new;
  $hbox->show;

  my $ic = Video::Menc::GUI::ImageClip->new (
    gtk_window => $win,
    filename   => $filename,
    changed_callback => sub {
      my %par = @_;
      my ($clip_type, $value) = @par{'type','value'};
      $clip_type =~ s/^clip/tc_$type/;
      my $old_value = $self->adjust_widgets
                    ->{$clip_type}
                    ->get_text;
      $self->adjust_widgets
           ->{$clip_type}
           ->set_text ($value);

      if ( $clip_type =~ /left/ or $clip_type =~ /right/ ) {
        my $width = $self->adjust_widgets->{"image_$type"}->image_width;
        $width -= $value-$old_value;
        $self->adjust_widgets->{"image_$type"}->set_image_width($width);
      } else {
        my $height = $self->adjust_widgets->{"image_$type"}->image_height;
        $height -= $value-$old_value;
        $self->adjust_widgets->{"image_$type"}->set_image_height($height);
      }
      $self->show_preview_labels ( type => $type );
    },
    no_clip => ($type ne 'crop'),
  );

  if ( $type eq 'crop' ) {
    $ic->set_clip_top    ($title->tc_crop_top);
    $ic->set_clip_bottom ($title->tc_crop_bottom);
    $ic->set_clip_left   ($title->tc_crop_left);
    $ic->set_clip_right  ($title->tc_crop_right);
  }

  $ic->calculate_knobs;

  $hbox->pack_start($ic->widget, 0, 1, 0);
  $vbox->pack_start($hbox, 0, 1, 0);

  $self->adjust_widgets->{"window_$type"} = $win;

  1;
}

sub rescale {
  my $self = shift; $self->trace_in;
  
  my $title = $self->selected_title;
  return 1 if not $title;
  
  my $widgets = $self->adjust_widgets;
  
  my $width = $title->width;
  my $height = $title->height;
  my $adjust_width = $title->adjust_width;
  my $adjust_height = $title->adjust_height;
      
  $width = $adjust_width if ($adjust_width);
  $height = $adjust_height if ($adjust_height);    
  
  my $rescale_width = undef;
  my $rescale_height = undef;
  
  my $string;
  $string = $widgets->{tc_rescale_width}->get_text;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;
  $rescale_width = $string if ($string ne "");  
  
  $string = $widgets->{tc_rescale_height}->get_text;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;
  $rescale_height = $string if ($string ne "");
  
  return 1 if ($rescale_height == undef and $rescale_width == undef);
  
  if ($rescale_height != undef and $rescale_width == undef) {
    $rescale_width = $rescale_height * $width / $height;
    $rescale_width = $rescale_width / 8 + 0.5;
    $rescale_width = sprintf("%d", $rescale_width);
    $rescale_width = $rescale_width * 8;
  }
  
  if ($rescale_height == undef and $rescale_width != undef) {
    $rescale_height = $rescale_width * $height / $width;
    $rescale_height = $rescale_height / 8 + 0.5;
    $rescale_height = sprintf("%d", $rescale_height);
    $rescale_height = $rescale_height * 8;      
  }  
  
  my $fw = $rescale_width / $width;
  my $fh = $rescale_height / $height;
  
  my $scalew = $title->tc_scale_width;
  my $scaleh = $title->tc_scale_height;
  my $cropt = $title->tc_crop_top;
  my $cropb = $title->tc_crop_bottom;
  my $cropl = $title->tc_crop_left;
  my $cropr = $title->tc_crop_right;
  
  my $recropt = $cropt * $fh;
  $recropt = sprintf("%d", $recropt);  
  my $recropb = $cropb * $fh;
  $recropb = sprintf("%d", $recropb);
  my $recropl = $cropl * $fw;
  $recropl = sprintf("%d", $recropl);  
  my $recropr = $cropr * $fw;
  $recropr = sprintf("%d", $recropr);
  
  $scalew = $rescale_width + $recropl + $recropr;
  $scaleh = $rescale_height + $recropt + $recropb;
  
  $widgets->{tc_scale_width}->set_text("${scalew}");
  $widgets->{tc_scale_height}->set_text("${scaleh}");
  $widgets->{tc_crop_top}->set_text("${recropt}");  
  $widgets->{tc_crop_bottom}->set_text("${recropb}");
  $widgets->{tc_crop_left}->set_text("${recropl}");  
  $widgets->{tc_crop_right}->set_text("${recropr}");  

  $self->make_previews;
  $self->show_preview_images;
  $self->get_sizes;
  $self->fill_with_values;      
}

1;
