#-----------------------------------------------------------------------
# Copyright (C) 2003 Boris Schfer <menc@noid-project> All Rights Reserved
# 
# This module is part of Video::Menc, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::Menc::GUI::Project;

use Carp;
use strict;

#---------------------------------------------------------------------
# Build Mencoder Tab
#---------------------------------------------------------------------

sub create_mencoder_tab {
  my $self = shift; $self->trace_in;

  my $vbox = Gtk::VBox->new;
  $vbox->set_border_width(5);
  $vbox->show;
  
  my $selected_title = $self->create_selected_title;
  $vbox->pack_start ( $selected_title, 0, 1, 0);

  my ($frame_hbox, $frame, $label, $entry, $hbox, $table,
      $button, $popup_menu, $popup, $item, %popup_entries,
      $storage_hbox);

  $frame = Gtk::Frame->new ("Adjust Mencoder Options");
  $frame->show;
  $vbox->pack_start ( $frame, 0, 1, 0);

  $frame_hbox = Gtk::HBox->new;
  $frame_hbox->set_border_width(5);
  $frame_hbox->show;
  $frame->add ($frame_hbox);

  # Left Table -------------------------------------------------

  $table = Gtk::Table->new ( 5, 2, 0 );
  $table->show;
  $table->set_row_spacings ( 10 );
  $table->set_col_spacings ( 10 );
  $frame_hbox->pack_start ($table, 0, 1, 0);

  my $row = -1;
  
  # Audio
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Audio");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $popup_menu = Gtk::Menu->new;
  $popup_menu->show;
  $popup = Gtk::OptionMenu->new;
  $popup->show;
  $popup->set_menu($popup_menu);
  $popup->set_usize(220,undef);

  %popup_entries = (
    0 => "No Sound",
    1 => "Automatic",
    2 => "MP3 128kBit/s",
    3 => "MP3 160kBit/s",
    4 => "Copy",
  );
  foreach my $key ( sort keys %popup_entries ) {
    $item = Gtk::MenuItem->new ($popup_entries{$key});
    $item->show;
    $popup_menu->append($item);
    $item->signal_connect (
      "select", sub {
        $self->project->set_oac($key);
        $self->update_bitrates;
      }, $key
    );
  }
  $table->attach_defaults ($popup, 1, 2, $row, $row+1);

  $self->adjust_widgets->{audio_popup} = $popup;

  # 2nd Audio
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("2nd Audio");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $popup_menu = Gtk::Menu->new;
  $popup_menu->show;
  $popup = Gtk::OptionMenu->new;
  $popup->show;
  $popup->set_menu($popup_menu);
  $popup->set_usize(220,undef);

  %popup_entries = (
    0 => "No Sound",
    1 => "Automatic",
    2 => "MP3 128kBit/s",
    3 => "MP3 160kBit/s",
    4 => "Copy",
  );
  foreach my $key ( sort keys %popup_entries ) {
    $item = Gtk::MenuItem->new ($popup_entries{$key});
    $item->show;
    $popup_menu->append($item);
    $item->signal_connect (
      "select", sub {
        $self->project->set_oac2($key);
        $self->update_bitrates;
      }, $key
    );
  }
  $table->attach_defaults ($popup, 1, 2, $row, $row+1);

  $self->adjust_widgets->{audio2_popup} = $popup;

  # Enable DivX Multipass Encoding
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("DivX 2-pass");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $hbox = Gtk::HBox->new;
  $hbox->show;
  my $radio_yes = Gtk::RadioButton->new ("Yes");
  $radio_yes->show;
  $hbox->pack_start($radio_yes, 0, 1, 0);
  my $radio_no = Gtk::RadioButton->new ("No", $radio_yes);
  $radio_no->show;
  $hbox->pack_start($radio_no, 0, 1, 0);
  
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);
  
  $self->adjust_widgets->{twopass_yes} = $radio_yes;
  $self->adjust_widgets->{twopass_no}  = $radio_no;
  
  $radio_yes->signal_connect (
    "clicked", sub {
      $self->project->set_twopass(1);
    }
  );
  $radio_no->signal_connect (
    "clicked", sub {
      $self->project->set_twopass(0);
    }
  );
  
  # VOP lavcdeint
  $row++;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Deinterlace (kerndeint)");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $hbox = Gtk::HBox->new;
  $hbox->show;  
  my $radio_no = Gtk::RadioButton->new ("No");
  $radio_no->show;
  $hbox->pack_start($radio_no, 0, 1, 0);
  my $radio_yes = Gtk::RadioButton->new ("Yes", $radio_no);
  $radio_yes->show;
  $hbox->pack_start($radio_yes, 0, 1, 0);
  
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);
  
  $self->adjust_widgets->{lavcdeint_yes} = $radio_yes;
  $self->adjust_widgets->{lavcdeint_no}  = $radio_no;

  $radio_yes->signal_connect (
    "clicked", sub {
      $self->project->set_vop_lavcdeint(1);
    }
  );
  $radio_no->signal_connect (
    "clicked", sub {
      $self->project->set_vop_lavcdeint(0);
    }
  );  
  
  # VOP field
  $row++;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Field");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $hbox = Gtk::HBox->new;
  $hbox->show;
  $radio_no = Gtk::RadioButton->new ("No");
  $radio_no->show;
  $hbox->pack_start($radio_no, 0, 1, 0);  
  my $radio_even = Gtk::RadioButton->new ("Even", $radio_no);
  $radio_even->show;
  $hbox->pack_start($radio_even, 0, 1, 0);
  my $radio_odd = Gtk::RadioButton->new ("Odd", $radio_no);
  $radio_odd->show;
  $hbox->pack_start($radio_odd, 0, 1, 0);
  
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  $self->adjust_widgets->{field_no}   = $radio_no;
  $self->adjust_widgets->{field_even} = $radio_even;
  $self->adjust_widgets->{field_odd}  = $radio_odd;

  $radio_no->signal_connect (
    "clicked", sub {
      $self->project->set_vop_field(-1);
    }
  );
  $radio_even->signal_connect (
    "clicked", sub {
      $self->project->set_vop_field(0);
    }
  );
  $radio_odd->signal_connect (
    "clicked", sub {
      $self->project->set_vop_field(1);
    }
  );    

  # VOP avsync
#  $row++;
#  $hbox = Gtk::HBox->new;
#  $hbox->show;
#  $label = Gtk::Label->new ("A/V Sync");
#  $label->show;
#  $hbox->pack_start($label, 0, 1, 0);
#  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);  
#
#  $hbox = Gtk::HBox->new;
#  $hbox->show;
#  $entry = Gtk::Entry->new;
#  $entry->show;
#  $entry->set_usize(80,undef);
#  $hbox->pack_start($entry, 0, 1, 0);
#  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);
#
#  $self->adjust_widgets->{avsync} = $entry;
#
#  $entry->signal_connect (
#    "changed", sub {
#      $self->project->set_vop_avsync( $entry->get_text );
#      my $tmp = $entry->get_text;
#    }
#  );

  # Playbutton
  $row++;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  
  my $button;  
  $button = Gtk::Button->new_with_label (" Play (with filters) ");
  $button->show;
  $hbox->pack_start ($button, 1, 1, 0);
  $button->signal_connect ("clicked",
    sub { $self->play_with_vop }
  );
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  # Separator

  my $sep = Gtk::VSeparator->new;
  $sep->set_usize(40,undef);
  $sep->show;
  $frame_hbox->pack_start ($sep, 0, 1, 0);
  
  # Right Table ------------------------------------------------
  
  $table = Gtk::Table->new ( 5, 2, 0 );
  $table->show;
  $table->set_row_spacings ( 10 );
  $table->set_col_spacings ( 10 );
  $frame_hbox->pack_start ($table, 0, 1, 0);

  $row = -1;

  # Video Length
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Video Length (sec)");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $hbox = Gtk::HBox->new;
  $hbox->show;
  $entry = Gtk::Entry->new;
  $entry->show;
  $entry->set_usize(80,undef);
  $hbox->pack_start($entry, 0, 1, 0);
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  $self->adjust_widgets->{video_length} = $entry;
  
  $entry->signal_connect (
    "changed", sub {
      $self->project->set_video_length( $entry->get_text );
      $self->update_bitrates;
    }
  );
  
  $label = Gtk::Label->new ("Default: ");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $label = Gtk::Label->new ("");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $self->adjust_widgets->{video_length_default} = $label;

  # Video Bitrate
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Video Bitrate (kBit/s)");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $hbox = Gtk::HBox->new;
  $hbox->show;
  $entry = Gtk::Entry->new;
  $entry->show;
  $entry->set_usize(80,undef);
  $hbox->pack_start($entry, 0, 1, 0);
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  $self->adjust_widgets->{video_bitrate} = $entry;
  
  $entry->signal_connect (
    "changed", sub {
      $self->project->set_video_bitrate( $entry->get_text );
      $self->update_bitrates;
    }
  );

  $label = Gtk::Label->new ("Default: ");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $label = Gtk::Label->new ("");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $self->adjust_widgets->{video_bitrate_default} = $label;
    
  # Audio Bitrate
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Audio Bitrate (kBit/s)");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $hbox = Gtk::HBox->new;
  $hbox->show;
  $entry = Gtk::Entry->new;
  $entry->show;
  $entry->set_usize(80,undef);
  $hbox->pack_start($entry, 0, 1, 0);
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  $self->adjust_widgets->{audio_bitrate} = $entry;

  $entry->signal_connect (
    "changed", sub {
      $self->project->set_audio_bitrate( $entry->get_text );
      $self->update_bitrates;
    }
  );
  
  $label = Gtk::Label->new ("Default: ");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $label = Gtk::Label->new ("");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $self->adjust_widgets->{audio_bitrate_default} = $label;

  # 2nd Audio Bitrate
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("2nd Audio Bitrate (kBit/s)");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $hbox = Gtk::HBox->new;
  $hbox->show;
  $entry = Gtk::Entry->new;
  $entry->show;
  $entry->set_usize(80,undef);
  $hbox->pack_start($entry, 0, 1, 0);
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  $self->adjust_widgets->{audio2_bitrate} = $entry;

  $entry->signal_connect (
    "changed", sub {
      $self->project->set_audio2_bitrate( $entry->get_text );
      $self->update_bitrates;
    }
  );
  
  $label = Gtk::Label->new ("Default: ");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $label = Gtk::Label->new ("");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  $self->adjust_widgets->{audio2_bitrate_default} = $label;

  # Target BPP
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  my $radio_target_bpp = Gtk::RadioButton->new ("Target BPP");
  $radio_target_bpp->show;
  $hbox->pack_start($radio_target_bpp, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);
  
  $radio_target_bpp->signal_connect (
    "clicked", sub {
      $self->project->set_use_bpp(1);
      $self->update_bitrates;
    }
  );

    $self->adjust_widgets->{radio_target_bpp} = $radio_target_bpp;
  
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $entry = Gtk::Entry->new;
  $entry->show;
  $entry->set_usize(80,undef);
  $hbox->pack_start($entry, 0, 1, 0);
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  $self->adjust_widgets->{target_bpp} = $entry;

  $entry->signal_connect (
    "changed", sub {
      $self->project->set_target_size( $entry->get_text );
      $self->update_bitrates;
    }
  );
    
  $label = Gtk::Label->new ("Default: ");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
    
  # bpp popup
  $popup_menu = Gtk::Menu->new;
  $popup_menu->show;
  $popup = Gtk::OptionMenu->new;
  $popup->show;
  $popup->set_menu($popup_menu);
  $popup->set_usize(70,undef);

  %popup_entries = (
    0.15 => "0.15",
    0.2  => "0.20",
    0.25 => "0.25",
  );

  foreach my $key ( sort keys %popup_entries ) {
    $item = Gtk::MenuItem->new ($popup_entries{$key});
    $item->show;
    $popup_menu->append($item);
    $item->signal_connect (
      "select", sub {
          $self->project->set_target_bpp_default($key);
        $self->update_bitrates;
      }, $key
    );
  }
  $hbox->pack_start($popup, 0, 1, 0);  
  
  $self->adjust_widgets->{bpp_popup} = $popup;

  # Target Size
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  my $radio_target_size = Gtk::RadioButton->new ("Target Size (MB)",
                                                 $radio_target_bpp);
  $radio_target_size->show;
  $hbox->pack_start($radio_target_size, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $radio_target_size->signal_connect (
    "clicked", sub {
      $self->project->set_use_bpp(0);
      $self->update_bitrates;
    }
  );

    $self->adjust_widgets->{radio_target_size} = $radio_target_size;

  $hbox = Gtk::HBox->new;
  $hbox->show;
  $entry = Gtk::Entry->new;
  $entry->show;
  $entry->set_usize(80,undef);
  $hbox->pack_start($entry, 0, 1, 0);
  $table->attach_defaults ($hbox, 1, 2, $row, $row+1);

  $self->adjust_widgets->{target_size} = $entry;

  $entry->signal_connect (
    "changed", sub {
      $self->project->set_target_size( $entry->get_text );
      $self->update_bitrates;
    }
  );
    
  $label = Gtk::Label->new ("Default: ");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
    
  # disc cnt popup
  $popup_menu = Gtk::Menu->new;
  $popup_menu->show;
  $popup = Gtk::OptionMenu->new;
  $popup->show;
  $popup->set_menu($popup_menu);
  $popup->set_usize(70,undef);

  %popup_entries = (
    1 => "one",
    2 => "two",
    3 => "three",
  );

  foreach my $key ( sort keys %popup_entries ) {
    $item = Gtk::MenuItem->new ($popup_entries{$key});
    $item->show;
    $popup_menu->append($item);
    $item->signal_connect (
      "select", sub {
        my $size = $key * $self->project->disc_size;
        $self->project->set_disc_count($key);
        $self->project->set_target_size_default($size);
        $self->update_bitrates;
      }, $key
    );
  }
  $hbox->pack_start($popup, 0, 1, 0);  
  
  $self->adjust_widgets->{disc_count_popup} = $popup;

  # disc size popup
  $popup_menu = Gtk::Menu->new;
  $popup_menu->show;
  $popup = Gtk::OptionMenu->new;
  $popup->show;
  $popup->set_menu($popup_menu);
  $popup->set_usize(130,undef);

  %popup_entries = (
    650 => "650 MB disc(s)",
    700 => "700 MB disc(s)",
  );

  foreach my $key ( sort keys %popup_entries ) {
    $item = Gtk::MenuItem->new ($popup_entries{$key});
    $item->show;
    $popup_menu->append($item);
    $item->signal_connect (
      "select", sub {        
        my $size = $key * $self->project->disc_count;
        $self->project->set_disc_size($key);
        $self->project->set_target_size_default($size);
        $self->update_bitrates;
      }, $key
    );
  }
  $hbox->pack_start($popup, 0, 1, 0);  
  
  $self->adjust_widgets->{disc_size_popup} = $popup;


  # Calculated Storage -----------------------------------------
  
  $frame = Gtk::Frame->new ("Calculations");
  $frame->show;
  $vbox->pack_start ( $frame, 0, 1, 0);

  $frame_hbox = Gtk::HBox->new;
  $frame_hbox->set_border_width(5);
  $frame_hbox->show;
  $frame->add ($frame_hbox);

  # Left Table -------------------------------------------------

  $table = Gtk::Table->new ( 5, 2, 0 );
  $table->show;
  $table->set_row_spacings ( 10 );
  $table->set_col_spacings ( 10 );
  $frame_hbox->pack_start ($table, 0, 1, 0);

  my $row = -1;

  # Video
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Video Size (MB):");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $label = Gtk::Label->new ("");
  $label->show;
  $label->set_justify('left');
  $hbox->pack_end($label, 0, 1, 0);

  $self->adjust_widgets->{label_video_size} = $label;

  # Audio
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("Audio Size (MB):");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $label = Gtk::Label->new ("");
  $label->show;
  $label->set_justify('left');  
  $hbox->pack_end($label, 0, 1, 0);

  $self->adjust_widgets->{label_audio_size} = $label;

  # 2nd Audio
  ++$row;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $label = Gtk::Label->new ("2nd Audio Size (MB):");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $label = Gtk::Label->new ("");
  $label->show;
  $label->set_justify('left');  
  $hbox->pack_end($label, 0, 1, 0);

  $self->adjust_widgets->{label_audio2_size} = $label;  

  # Separator
  ++$row;
  my $sep = Gtk::HSeparator->new;
  $sep->set_usize(40,undef);
  $sep->show;
  $table->attach_defaults ($sep, 0, 1, $row, $row+1);

  # Sum
  ++$row;
  $label = Gtk::Label->new ("Total Size (MB):");
  $label->show;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);

  $label = Gtk::Label->new ("");
  $label->show;  
  $label->set_justify('left');
  $hbox->pack_end($label, 0, 1, 0);

  $self->adjust_widgets->{label_total_size} = $label;

  # Separator

  $sep = Gtk::VSeparator->new;
  $sep->set_usize(40,undef);
  $sep->show;
  $frame_hbox->pack_start ($sep, 0, 1, 0);
  
  # Right Table ------------------------------------------------
  
  $table = Gtk::Table->new ( 5, 2, 0 );
  $table->show;
  $table->set_row_spacings ( 10 );
  $table->set_col_spacings ( 10 );
  $frame_hbox->pack_start ($table, 0, 1, 0);

  $row = -1;

  # Bitrates
  ++$row;
  $label = Gtk::Label->new ("");
  $label->show;
  $hbox = Gtk::HBox->new;
  $hbox->show;
  $hbox->pack_start($label, 0, 1, 0);
  $table->attach_defaults ($hbox, 0, 1, $row, $row+1);  

  $self->adjust_widgets->{label_bitrates} = $label;
  
  # Filename  
  $frame = Gtk::Frame->new ("Project");
  $frame->show;
  $vbox->pack_start ( $frame, 0, 1, 0);

  my $frame_vbox = Gtk::VBox->new;
  $frame_vbox->set_border_width(5);
  $frame_vbox->show;
  $frame->add ($frame_vbox);  
  
  $frame_hbox = Gtk::HBox->new;
  $frame_hbox->set_border_width(5);
  $frame_hbox->show;
  $frame_vbox->pack_start($frame_hbox, 0, 1, 0);
  
  my $label = Gtk::Label->new(" Output Filename ");
  $label->show;
  $frame_hbox->pack_start($label, 0, 1, 0);

  my $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->set_usize(500, undef);
  $entry->show;
  $entry->signal_connect ("changed",
    sub { $self->use_avifile }
  );
  $self->adjust_widgets->{avifilename} = $entry;

  $frame_hbox->pack_start($entry, 0, 1, 0);
    
  my $button = Gtk::Button->new_with_label (" Browse... ");
  $button->show;
  $frame_hbox->pack_start ($button, 1, 1, 0);
  $button->signal_connect ("clicked",
    sub { $self->browse_avifile }
  );
  
  # Split
  $frame_hbox = Gtk::HBox->new;
  $frame_hbox->set_border_width(5);
  $frame_hbox->show;
  $frame_vbox->pack_start($frame_hbox, 0, 1, 0);
  
  my $label = Gtk::Label->new(" Cut Avi-File ");
  $label->show;
  $frame_hbox->pack_start($label, 0, 1, 0);

  my $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->show;
  $entry->signal_connect ("changed",
    sub { $self->cut_changed }
  );
  $self->adjust_widgets->{cutstart} = $entry;

  $frame_hbox->pack_start($entry, 0, 1, 0);

  $label = Gtk::Label->new(" - ");
  $label->show;
  $frame_hbox->pack_start($label, 0, 1, 0);

  $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->show;
  $entry->signal_connect ("changed",
    sub { $self->cut_changed }
  );
  $self->adjust_widgets->{cutend} = $entry;

  $frame_hbox->pack_start($entry, 0, 1, 0);  

  $label = Gtk::Label->new("");
  $label->show;
  $frame_hbox->pack_start($label, 0, 1, 0);
  $self->adjust_widgets->{cutinfo} = $label;
  
  # Buttons
  $frame_hbox = Gtk::HBox->new (1);
  $frame_hbox->set_border_width(5);
  $frame_hbox->show;
  $frame_vbox->pack_start($frame_hbox, 0, 1, 0);  

  my $button;  
  $button = Gtk::Button->new_with_label (" Save Project ");
  $button->show;
  $frame_hbox->pack_start ($button, 1, 1, 0);
  $button->signal_connect ("clicked",
    sub { $self->save_project }
  );
  
  return $vbox;
}

sub update_bitrates {
  my $self = shift; $self->trace_in;

  my $project = $self->project;
  my $widgets = $self->adjust_widgets;

  my $video_length_default = undef;
  my $video_bitrate_default = undef;
  my $audio_bitrate_default = undef;
  my $audio2_bitrate_default = undef;
  
  my $title = $self->selected_title;
  if ($title) {
    $video_length_default = $title->runtime;
    
    my $oac = $project->oac;
    my $oac2 = $project->oac2;
      
    $audio_bitrate_default = 0;

    my $audio_channel = $title->audio_channel;
    my $audio2_channel = $title->audio2_channel;
    if ( $title->audio_tracks ) {
      if ( @{$title->audio_tracks} >= 1 ) {
        if ( $audio_channel >= 0 ) {          
          if ($oac == 1) {
            my $audio = $title->audio_tracks->[$audio_channel];
            my $format = uc($audio->{format});
            
            if ($format =~ /^AC3.*/) {
                $audio_bitrate_default = $audio->{bitrate} / 1000;
            } else {
                $audio_bitrate_default = 160;
            }
          }
          $audio_bitrate_default = 128 if ($oac == 2);
          $audio_bitrate_default = 160 if ($oac == 3);
          $audio_bitrate_default = 0 if ($oac == 4);
        }
        if ( $audio2_channel >= 0 ) {          
          if ($oac2 == 1) {
            my $audio = $title->audio_tracks->[$audio2_channel];
            my $format = uc($audio->{format});
            
            if ($format =~ /^AC3.*/) {
                $audio2_bitrate_default = $audio->{bitrate} / 1000;
            } else {
                $audio2_bitrate_default = 160;
            }
          }
          $audio2_bitrate_default = 128 if ($oac2 == 2);
          $audio2_bitrate_default = 160 if ($oac2 == 3);
          $audio2_bitrate_default = 0 if ($oac2 == 0);
        }        
      }
    }
  }
  
  $project->set_video_length_default($video_length_default);
  $project->set_audio_bitrate_default($audio_bitrate_default);
  $project->set_audio2_bitrate_default($audio2_bitrate_default);
  
  if ($video_length_default == undef) {
    $widgets->{video_length_default}->set_text("-undefined-");
  } else {
    $widgets->{video_length_default}->set_text($video_length_default." sec");
  }
  
  if ($audio_bitrate_default == undef) {
    $widgets->{audio_bitrate_default}->set_text("-undefined-");
  } else {
    $widgets->{audio_bitrate_default}->set_text($audio_bitrate_default." kbits/s");
  }
  
  if ($audio2_bitrate_default == undef) {
    $widgets->{audio2_bitrate_default}->set_text("-undefined-");
  } else {
    $widgets->{audio2_bitrate_default}->set_text($audio2_bitrate_default." kbits/s");
  }  
  
  my $oac = $project->oac;
  my $oac2 = $project->oac2;
  my $target_bpp_default = $project->target_bpp_default;
  my $target_size_default = $project->target_size_default;
  
  my $video_length = undef; 
  my $audio_bitrate = undef;
  my $audio2_bitrate = undef;
  my $target_bpp = undef;
  my $target_size = undef;
  
  my $string;
  $string = $widgets->{video_length}->get_text;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;
  $video_length = $string if ($string ne "");
  
  $string = $widgets->{audio_bitrate}->get_text;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;  
  $audio_bitrate = $string if ($string ne "");
  
  $string = $widgets->{audio2_bitrate}->get_text;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;  
  $audio2_bitrate = $string if ($string ne "");  
  
  $string = $widgets->{target_bpp}->get_text;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;    
  $target_bpp = $string if ($string ne "");  
  
  $string = $widgets->{target_size}->get_text;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;    
  $target_size = $string if ($string ne "");

  $video_length = $video_length_default if (not defined $video_length);
  $audio_bitrate = $audio_bitrate_default if (not defined $audio_bitrate); 
  $audio2_bitrate = $audio2_bitrate_default if (not defined $audio2_bitrate);
  $target_bpp = $target_bpp_default if (not defined $target_bpp);
  $target_size = $target_size_default if (not defined $target_size);  
  
  my $use_bpp = $project->use_bpp;
  
  if ($use_bpp) {
      if ($video_length > 0) {
        if ($title) {
            my $width = $title->width;
        my $height = $title->height;
        my $fps = $title->fps;
        my $adjust_width = $title->adjust_width;
        my $adjust_height = $title->adjust_height;
        
        $width = $adjust_width if ($adjust_width);
          $height = $adjust_height if ($adjust_height);
        
        $video_bitrate_default = 
            ($target_bpp * $width * $height * $fps) / 1000;
        $video_bitrate_default = sprintf("%d", $video_bitrate_default);  
      }
    }    
  } else {
      if ($video_length > 0) {
        my $size = $target_size * 1024 * 1024;
        my $audio = ($audio_bitrate+$audio2_bitrate) * 1024 * 
                    $video_length / 8;
        $video_bitrate_default = (($size - $audio)*8)/($video_length * 1015);
        $video_bitrate_default = sprintf("%d", $video_bitrate_default);
      }
  }  

  if (not $video_bitrate_default) {
    $widgets->{video_bitrate_default}->set_text("-undefined-");
  } else {
    my $bpp;
    my $title = $self->selected_title;
    if ($title) {      
      my $width = $title->width;
      my $height = $title->height;
      my $fps = $title->fps;
      my $adjust_width = $title->adjust_width;
      my $adjust_height = $title->adjust_height;
      
      $width = $adjust_width if ($adjust_width);
      $height = $adjust_height if ($adjust_height);
          
      $bpp = ($video_bitrate_default * 1000) / ($width * $height * $fps);
      $bpp = sprintf(" (%.2f bpp)", $bpp);
    }
    
    $widgets->{video_bitrate_default}->
      set_text($video_bitrate_default." kbits/s".$bpp);
  }
  
  $project->set_video_bitrate_default($video_bitrate_default);

  my $video_bitrate = undef;
  $string = $widgets->{video_bitrate}->get_text;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;  
  $video_bitrate = $string if ($string ne "");  
  
  $video_bitrate = $video_bitrate_default if (not defined $video_bitrate);  
  
  if ($video_bitrate > 0) {
    # Sizes
    my $asize = $audio_bitrate * 1024 * $video_length / 8 / 1024 / 1024;
    $asize = sprintf("%d", $asize);
    my $a2size = $audio2_bitrate * 1024 * $video_length / 8 / 1024 / 1024;
    $a2size = sprintf("%d", $a2size);    
    my $vsize = $video_bitrate * 1010 * $video_length / 8 / 1024 / 1024;
    $vsize = sprintf("%d", $vsize);
    my $ssize = $asize + $a2size + $vsize;
  
    $widgets->{label_video_size}->set_text($vsize);   
    $widgets->{label_audio_size}->set_text($asize);
    $widgets->{label_audio2_size}->set_text($a2size);  
    $widgets->{label_total_size}->set_text($ssize);
    
    # Bitrates
    my $title = $self->selected_title;
    if ($title) {
      my $width = $title->width;
      my $height = $title->height;
      my $fps = $title->fps;      
      my $adjust_width = $title->adjust_width;
      my $adjust_height = $title->adjust_height;
      my $disc_size = $project->disc_size;
      my $use_bpp = $project->use_bpp;
      
      $width = $adjust_width if ($adjust_width);
      $height = $adjust_height if ($adjust_height);
      
      my $audio = ($audio_bitrate + $audio2_bitrate) * 1024 * 
                  $video_length / 8;
      my $bitrate = "";
      
      
      for (my $i=0; $i<4; $i++) {
        my $w;
        $w = 768 if ($i == 0);
        $w = 720 if ($i == 1);
        $w = 704 if ($i == 2);
        $w = 640 if ($i == 3);
        
        my $h = $w / $width * $height;
        $h = $h / 8 + 0.5;
        $h = sprintf("%d", $h);
        $h = $h * 8;
        
        $bitrate = $bitrate."$w x $h: ";
        
        if ($use_bpp) {
          my $asize = ($audio_bitrate + $audio2_bitrate) * 1024 * 
                      $video_length / 8 / 1024 / 1024;
             $asize = sprintf("%d", $asize);
          
          for (my $j=0.15; $j<=0.25; $j+=0.05) {
              my $rate = ($j * $w * $h * $fps) / 1000;
            $rate = sprintf("%4d", $rate);
            
            my $vsize = $rate * 1010 *$video_length / 8 / 1024 / 1024;
            $vsize = sprintf("%4d", $vsize);
            my $size = $vsize + $asize;
            
            $bitrate = $bitrate."${j}bpp: ${rate}kbits/s, ${size}MB";
            $bitrate = $bitrate."   " if ($j < 0.25);
          } 
        } else {
          for (my $j=1; $j<4; $j++) {
            my $rate = (($disc_size*1024*1024 * $j - $audio) * 8) /
                      ($video_length * 1015);
            $rate = sprintf("%4d", $rate);
          
            my $bpp = ($rate * 1000) / ($w * $h * $fps);
            $bpp = sprintf("%5.2f", $bpp);
          
            my $s = "'s" if ($j > 1);
          
            $bitrate = $bitrate."${j}CD${s}: ${rate}kbits/s, ${bpp}bpp";
            $bitrate = $bitrate."   " if ($j < 3);
          }
        }
        $bitrate = $bitrate."\n";
      }
      $widgets->{label_bitrates}->set_text($bitrate);
    } else {
      $widgets->{label_bitrates}->set_text("");
    }
      
  } else {
    $widgets->{label_video_size}->set_text("-undefined-");   
    $widgets->{label_audio_size}->set_text("-undefined-");  
    $widgets->{label_audio2_size}->set_text("-undefined-");
    $widgets->{label_total_size}->set_text("-undefined-");
    $widgets->{label_bitrates}->set_text("");  
  }
  
  $self->cut_changed;
}

sub init_mencoder_values {
  my $self = shift; $self->trace_in;

  my $project = $self->project;
  my $widgets = $self->adjust_widgets;

  my $audio = $project->oac;
  my $audio2 = $project->oac2;
  my $twopass = $project->twopass;
  my $vop_lavcdeint = $project->vop_lavcdeint;
  my $vop_field = $project->vop_field;
    
  $widgets->{audio_popup}->set_history ($audio);
  $widgets->{audio2_popup}->set_history ($audio2);
  
  $widgets->{lavcdeint_yes}->set_active($vop_lavcdeint);
  $widgets->{lavcdeint_no}->set_active(!$vop_lavcdeint);  

  if ($vop_field == -1) {
    $widgets->{field_no}->set_active(1);
    $widgets->{field_even}->set_active(0);
    $widgets->{field_odd}->set_active(0);    
  }
  if ($vop_field == 0) {
    $widgets->{field_no}->set_active(0);
    $widgets->{field_even}->set_active(1);
    $widgets->{field_odd}->set_active(0);    
  }
  if ($vop_field == 1) {
    $widgets->{field_no}->set_active(0);
    $widgets->{field_even}->set_active(0);
    $widgets->{field_odd}->set_active(1);    
  }
  
  my $target_bpp = $project->target_bpp_default;
  $widgets->{bpp_popup}->set_history (0) if ($target_bpp == 0.15);
  $widgets->{bpp_popup}->set_history (1) if ($target_bpp == 0.2);
  $widgets->{bpp_popup}->set_history (2) if ($target_bpp == 0.25);
  
  my $use_bpp = $project->use_bpp;
  if ($use_bpp) {
    $widgets->{radio_target_bpp}->set_active(1);
    $widgets->{radio_target_size}->set_active(0);
  } else {
    $widgets->{radio_target_bpp}->set_active(0);
    $widgets->{radio_target_size}->set_active(1);  
  }      
  
  my $disc_count = $project->disc_count;
  my $disc_size = $project->disc_size;    
  my $size = $disc_count * $disc_size;
  $widgets->{disc_count_popup}->set_history ($disc_count-1);
  $widgets->{disc_size_popup}->set_history ($disc_size==650 ? 0 : 1);
  $project->set_target_size_default($size);  
  
  $self->update_bitrates;
}  

sub browse_avifile {
  my $self = shift;
  my $widgets = $self->adjust_widgets;

  my $filename = $widgets->{avifilename}->get_text;
  $filename =~ s/^\s+//;
  $filename =~ s/\s+$//;
  
  $self->show_file_dialog (
    dir      => ".",
    filename => "$filename",
    cb       => sub {
      my $filename = $_[0];
      $filename =~ s/^\s+//;
      $filename =~ s/\s+$//;      
      $self->set_avifile ( filename => $filename );
    },
  );
}

sub use_avifile {
  my $self = shift;
  my $widgets = $self->adjust_widgets;

  my $filename = $widgets->{avifilename}->get_text;
  $filename =~ s/^\s+//;
  $filename =~ s/\s+$//;
  
  $self->project->set_avifile($filename);
}

sub set_avifile {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($filename) = @par{'filename'};

  my $widgets = $self->adjust_widgets;
  $widgets->{avifilename}->set_text ( $filename );
  $self->project->set_avifile($filename);
}

sub cut_changed {
  my $self = shift; $self->trace_in;
  my $widgets = $self->adjust_widgets;
  
  my $start = $widgets->{cutstart}->get_text;
  my $end = $widgets->{cutend}->get_text;
  my $video_length = $widgets->{video_length}->get_text;
  my $video_length_default = $self->project->video_length_default;

  $video_length = $video_length_default if ($video_length eq "");

  $end = $video_length + $end if ($end < 0);
  $end = undef if ($end < 0);
  $end = undef if ($end <= $start);
  
  my $info;
    if (($start eq "") || ($end eq "")) {
    $info = "";
    $self->project->set_cut_start(undef);
    $self->project->set_cut_end(undef);
  } else {      
    my $h = int($start/3600);
    my $m = int(($start-$h*3600)/60);
    my $s = $start-$h*3600-$m*60;
    my $h2 = int($end/3600);
    my $m2 = int(($end-$h2*3600)/60);
    my $s2 = $end-$h2*3600-$m2*60;    
    
    $info = sprintf("   %d - %d  [%02d:%02d:%02d - %02d:%02d:%02d]", 
                    $start, $end, $h, $m, $s, $h2, $m2, $s2);
    $self->project->set_cut_start($start);
    $self->project->set_cut_end($end);    
  }
  $widgets->{cutinfo}->set_text ( $info );
}

sub play_with_vop {
  my $self = shift;

  my $title = $self->selected_title;
  return 1 if not $title;
  
  my $project = $self->project;
  
  my $width = $title->width;
  my $height = $title->height;
  my $scale_width = $title->tc_scale_width;
  my $scale_height = $title->tc_scale_height;
  my $crop_top = $title->tc_crop_top;
  my $crop_bottom = $title->tc_crop_bottom;
  my $crop_left = $title->tc_crop_left;
  my $crop_right = $title->tc_crop_right;
    
  if (not $scale_width and $scale_height) {
    $scale_width = $scale_height / $height * $width;
    $scale_width = $scale_width / 8 + 0.5;
    $scale_width = sprintf("%d", $scale_width);
    $scale_width = $scale_width * 8;    
  }
  
  if (not $scale_height and $scale_width) {
    $scale_height = $scale_width / $width * $height;
    $scale_height = $scale_height / 8 + 0.5;
    $scale_height = sprintf("%d", $scale_height);
    $scale_height = $scale_height * 8;        
  }
      
  my $adjust_width = $title->adjust_width;
  my $adjust_height = $title->adjust_height;
  $width = $adjust_width if ($adjust_width);
  $height = $adjust_height if ($adjust_height);
        
  my $lavcdeint = $project->vop_lavcdeint;
  my $field = $project->vop_field;
  #my $avsync = $project->vop_avsync;
#  my $avsync = $widgets->{avsync}->get_text;
  
  my $subtitle = $title->subtitle;
  
  my $menc_vop = "";
  
  # VOP
  if ($field >= 0) {
    my $vop_field;
    $vop_field = "field=0" if ($field == 0);  
    $vop_field = "field=1" if ($field == 1);
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop.$vop_field;    
  }
  
  if ($lavcdeint) {
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop."kerndeint";
  }

  if ($scale_width and $scale_height) {
    my $vop_scale = "scale=".$scale_width.":".$scale_height;
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop.$vop_scale;
  }   

  if ($crop_top or $crop_bottom or $crop_left or $crop_right) {
    my $vop_crop = "crop=".$width.":".$height.":".$crop_left.":".$crop_top;
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop.$vop_crop;  
  }

  if ($subtitle >= 0) {
    my $vop_expand = "expand=0:0:0:0:1";
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop.$vop_expand;
    
    $subtitle = "-sid $subtitle -spuaa 2 -spualign 2 -subpos 95";
  }
  $subtitle = "" if ($subtitle == -1);

#  if ($avsync > 0) {
#    my $vop_avsync;
#    $vop_avsync = "holdv=$avsync";
#    $menc_vop = $menc_vop."," if ($menc_vop ne "");
#    $menc_vop = $menc_vop.$vop_avsync;
#  }
  
  if ($menc_vop ne "") {
    $menc_vop = "-vf ".$menc_vop;
  }
  my $command = $title->precmd." mplayer -osdlevel 3 ".
                $title->audiocmd." ".$menc_vop." ".$subtitle." ".
		$title->postcmd." &";

  printf $command."\n";
  system ($command);
  
  1;
}

sub save_project {  
  my $self = shift; $self->trace_in;
  my $widgets = $self->adjust_widgets;

  my $title = $self->selected_title;
  return 1 if not $title;    

  my $avifile = $widgets->{avifilename}->get_text;
  $avifile =~ s/^\s+//;
  $avifile =~ s/\s+$//;

  my $projfile = $avifile;
  
  $projfile =~ s/\.[^\.]+$//;
  my $projfile2 = $projfile.".menc";
  
  $projfile2 = $projfile."_2.menc" if ($projfile2 eq $avifile);

  $self->show_file_dialog (
    dir      => ".",
    filename => "$projfile2",
    cb       => sub {
      my $filename = $_[0];
      $filename =~ s/^\s+//;
      $filename =~ s/\s+$//;      
      $self->save_project2 ( filename => $filename );
    },
  );  
}

sub save_project2 {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($filename) = @par{'filename'};

  my $title = $self->selected_title;
  return 1 if not $title;
  my $project = $self->project;  
  my $widgets = $self->adjust_widgets;

  my $avifile = $self->project->avifile;
  my $precmd = $title->precmd;
  my $postcmd = $title->postcmd;
  my $audiocmd = $title->audiocmd;
  my $audiocmd2 = $title->audiocmd2;
  $precmd =~ s/^\s+//;
  $precmd =~ s/\s+$//;
  $postcmd =~ s/^\s+//;
  $postcmd =~ s/\s+$//;
  $audiocmd =~ s/^\s+//;
  $audiocmd =~ s/\s+$//;
  $audiocmd2 =~ s/^\s+//;
  $audiocmd2 =~ s/\s+$//;
  
  my $width = $title->width;
  my $height = $title->height;
  my $scale_width = $title->tc_scale_width;
  my $scale_height = $title->tc_scale_height;
  my $crop_top = $title->tc_crop_top;
  my $crop_bottom = $title->tc_crop_bottom;
  my $crop_left = $title->tc_crop_left;
  my $crop_right = $title->tc_crop_right;
  my $subtitle = $title->subtitle;
    
  if (not $scale_width and $scale_height) {
    $scale_width = $scale_height / $height * $width;
    $scale_width = $scale_width / 8 + 0.5;
    $scale_width = sprintf("%d", $scale_width);
    $scale_width = $scale_width * 8;    
  }
  
  if (not $scale_height and $scale_width) {
    $scale_height = $scale_width / $width * $height;
    $scale_height = $scale_height / 8 + 0.5;
    $scale_height = sprintf("%d", $scale_height);
    $scale_height = $scale_height * 8;        
  }
      
  my $adjust_width = $title->adjust_width;
  my $adjust_height = $title->adjust_height;
  $width = $adjust_width if ($adjust_width);
  $height = $adjust_height if ($adjust_height);
        
  my $oac = $project->oac;
  my $oac2 = $project->oac2;
  my $twopass = $project->twopass;
  my $lavcdeint = $project->vop_lavcdeint;
  my $field = $project->vop_field;
  #my $avsync = $project->vop_avsync;
#  my $avsync = $widgets->{avsync}->get_text;
  
  my $video_bitrate_default = $project->video_bitrate_default;
  my $video_bitrate = undef;
  my $string = $widgets->{video_bitrate}->get_text;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;  
  $video_bitrate = $string if ($string ne "");  
  $video_bitrate = $video_bitrate_default if (not defined $video_bitrate);
  
  my $menc_audio = "";
  my $menc_audio2 = "";
  my $menc_video = "";
  my $menc_vpass = "";
  my $menc_vop = "";
  my $menc_output = "";
  
  # Audio
  my $has_audio = 0;
  my $audio_was_ac3 = 0;
  my $audio_is_ac3 = 0;
  my $audio_mp3_bitrate = 0;
  if (($title->audio_tracks) && ($oac > 0)) {
    my $audio_channel = $title->audio_channel;

    $has_audio = 1;

    if ($audio_channel >= 0) {
      my $audio = $title->audio_tracks->[$audio_channel];
      my $audio_format = uc($audio->{format});
      
      $audio_was_ac3 = 1 if ($audio_format =~ /^AC3.*/);        
    
      if ($oac == 1) {
        if ($audio_format =~ /^AC3.*/) {
          $menc_audio = "-oac copy";
          $audio_is_ac3 = 1;
        } else {
          $menc_audio = "-oac mp3lame -lameopts cbr:br=160:aq=2";
          $audio_mp3_bitrate = 160;
        }
      } elsif ($oac == 2) {
        $menc_audio = "-oac mp3lame -lameopts cbr:br=128:aq=2"; 
        $audio_mp3_bitrate = 128;
      } elsif ($oac == 3) {
        $menc_audio = "-oac mp3lame -lameopts cbr:br=160:aq=2"; 
        $audio_mp3_bitrate = 160;
      } elsif ($oac == 4) {
        $menc_audio = "-oac copy"; 
        $audio_mp3_bitrate = 0;
      }
    }  
  } else {
    $menc_audio = "";
    $audiocmd = "-nosound";  
  }

  # Audio2
  my $has_audio2 = 0;
  my $audio2_was_ac3 = 0;
  my $audio2_is_ac3 = 0;
  my $audio2_mp3_bitrate = 0;
  if (($title->audio_tracks) && ($oac2 > 0)) {
    my $audio_channel = $title->audio2_channel;
      
    $has_audio2 = 1;

    if ($audio_channel >= 0) {
      my $audio = $title->audio_tracks->[$audio_channel];
      my $audio_format = uc($audio->{format});    
        
      $audio2_was_ac3 = 1 if ($audio_format =~ /^AC3.*/);
            
      if ($oac2 == 1) {          
        if ($audio_format =~ /^AC3.*/) {
          $menc_audio2 = "-oac copy";
          $audio2_is_ac3 = 1;
        } else {
          $menc_audio2 = "-oac mp3lame -lameopts cbr:br=160:aq=2";
          $audio2_mp3_bitrate = 160;
        }
      } elsif ($oac2 == 2) {
          $menc_audio2 = "-oac mp3lame -lameopts cbr:br=128:aq=2"; 
          $audio2_mp3_bitrate = 128;
      } elsif ($oac2 == 3) {
          $menc_audio2 = "-oac mp3lame -lameopts cbr:br=160:aq=2"; 
          $audio2_mp3_bitrate = 160;
      } elsif ($oac2 == 4) {
          $menc_audio2 = "-oac copy"; 
          $audio2_mp3_bitrate = 0;
      }
    }  
  } else {
    $menc_audio2 = "";
    $audiocmd2 = "-nosound";
  }
  
  # Video
  $menc_video = "-ovc lavc -lavcopts vcodec=mpeg4:vbitrate=".
  $video_bitrate.":vhq:keyint=250";
          
  # VPass
  $menc_vpass = ":vpass=1:turbo" if ($twopass);
  
  # VOP
  if ($field >= 0) {
    my $vop_field;
    $vop_field = "field=0" if ($field == 0);  
    $vop_field = "field=1" if ($field == 1);
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop.$vop_field;    
  }
  
  if ($lavcdeint) {
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop."kerndeint";
  }  
  
  if ($scale_width and $scale_height) {
    my $vop_scale = "scale=".$scale_width.":".$scale_height;
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop.$vop_scale;
  }  
  
  if ($crop_top or $crop_bottom or $crop_left or $crop_right) {
    my $vop_crop = "crop=".$width.":".$height.":".$crop_left.":".$crop_top;
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop.$vop_crop;  
  }  
  
  if ($subtitle >= 0) {
    my $vop_expand = "expand=0:0:0:0:1";
    $menc_vop = $menc_vop."," if ($menc_vop ne "");
    $menc_vop = $menc_vop.$vop_expand;
    
    $subtitle = "-sid $subtitle -spuaa 2 -spualign 2 -subpos 95";
  }  
  $subtitle = "" if ($subtitle < 0);
  
  #$menc_vop = $menc_vop."," if ($menc_vop ne ""); 
  #$menc_vop = $menc_vop."harddup";
  
#  if ($avsync > 0) {
#    my $vop_avsync;
#    $vop_avsync = "holdv=$avsync";
#    $menc_vop = $menc_vop."," if ($menc_vop ne "");
#    $menc_vop = $menc_vop.$vop_avsync;
#  }
  
  if ($menc_vop ne "") {
    $menc_vop = "-vf ".$menc_vop;
  }
  
  # Output
  $menc_output = "-o \"".$avifile."\"";
  
  # CMD
  my $cmd;

  if ($twopass) {
    $cmd = $precmd." mencoder -nosound -ffourcc DIVX ".$menc_video.$menc_vpass.
           " ".$menc_vop." ".$subtitle." ".$menc_output." ".$postcmd;
    $cmd =~ s/^\s+//;
    $cmd =~ s/\s+$//;
  } else {
    $cmd = $precmd." mencoder ".$menc_audio." -ffourcc DIVX ".$menc_video.$menc_vpass.
           " ".$menc_vop." ".$audiocmd." ".$subtitle." ".$menc_output." ".$postcmd;
    $cmd =~ s/^\s+//;
    $cmd =~ s/\s+$//;
  }

  if ($twopass) {
    $menc_vpass = ":vpass=2";
    
    my $cmd2 = $precmd." mencoder ".$menc_audio." -ffourcc DIVX ".$menc_video.$menc_vpass.
               " ".$menc_vop." ".$audiocmd." ".$subtitle." ".$menc_output." ".$postcmd;
    $cmd2 =~ s/^\s+//;
    $cmd2 =~ s/\s+$//;
    
    $cmd = $cmd."\n".$cmd2;
    
  }   
  
  if (($has_audio2) && ($audiocmd2 ne "-nosound")) {
    my $cmd3 = $precmd." mencoder ".$menc_audio2." -ovc frameno ".$audiocmd2." -o \"".
               $avifile.".audio2\" ".$postcmd;
    $cmd3 =~ s/^\s+//;
    $cmd3 =~ s/\s+$//;
    
    $cmd = $cmd."\n".$cmd3;
    
    my $merge = "avimerge -i \"".$avifile."\" -o \"".$avifile.".tmp\" -p \"".
                $avifile.".audio2\" -a 2";
    $cmd = $cmd."\n".$merge;

    $cmd = $cmd."\nrm -f \"".$avifile.".audio2\"";
    $cmd = $cmd."\nmv \"".$avifile.".tmp\" \"".$avifile."\"";
  }  
  
  my $cut_start = $project->cut_start;
  my $cut_end = $project->cut_end;
  
  if (($cut_start ne "") && ($cut_end ne "")) {
    my $h = int($cut_start/3600);
    my $m = int(($cut_start-$h*3600)/60);
    my $s = $cut_start-$h*3600-$m*60;
    my $h2 = int($cut_end/3600);
    my $m2 = int(($cut_end-$h2*3600)/60);
    my $s2 = $cut_end-$h2*3600-$m2*60;    
    my $chunks = sprintf("%02d:%02d:%02d-%02d:%02d:%02d", 
                         $h, $m, $s, $h2, $m2, $s2);
            
    my $split = "avisplit -i \"".$avifile."\" -o \"".$avifile.".split\" ".
                "-t $chunks";
  
    $cmd = $cmd."\n".$split;
    $cmd = $cmd."\nmv \"".$avifile.".split-0000\" \"".$avifile."\"";
  }  
  
  # Write to File
  open(outfile, ">".$filename);

  print outfile "#!/bin/bash\n";
  print outfile "\n";
  print outfile $cmd."\n";

  close(outfile);
  
  chmod 0755, $filename;
}

1;
